// Author: Adrian Kuegel
// Date: August 29th, 2011
// Algorithm: greedy
// Complexity: O(n * m)

#include <stdio.h>
#include <assert.h>
#include <queue>
using namespace std;

char grid[1000][1001];
int dx[4] = {1,0,-1,0};
int dy[4] = {0,1,0,-1};
int n, m;
int deg_left_right[1000][1000], deg_top_bottom[1000][1000];
bool has_left_right[1000][1000], has_top_bottom[1000][1000];

int main() {
	int tc;
	scanf("%d", &tc);
	while(tc--) {
		scanf("%d %d", &n, &m);
		for (int i=0; i<n; ++i)
			scanf("%s", grid[i]);
		queue<int> Q, Q2;
		for (int i=0; i<n; ++i)
			for (int j=0; j<m; ++j) {
				deg_left_right[i][j] = deg_top_bottom[i][j] = 0;
				has_left_right[i][j] = has_top_bottom[i][j] = false;
				if (grid[i][j] == '.')
					continue;
				for (int d=0; d<4; ++d) {
					int tx = i + dx[d];
					int ty = j + dy[d];
					if (tx >= 0 && tx < n && ty >= 0 && ty < m && grid[tx][ty] != '.' && grid[tx][ty] != grid[i][j]) {
						if (d&1)
							++deg_left_right[i][j];
						else
							++deg_top_bottom[i][j];
					}
				}
				if (grid[i][j] == 'W') {
					if (deg_top_bottom[i][j]+deg_left_right[i][j] == 1)
						Q.push(i * m + j);
				}
				else if (deg_left_right[i][j] == 1 || deg_top_bottom[i][j] == 1)
					Q.push(i * m + j);
			}
		while(true) {
			// as long as there is a W with only one B neighbor or a B with only one W in left-right
			// or top-bottom direction, assign them to each other
			while(!Q.empty()) {
				int x = Q.front() / m;
				int y = Q.front() % m;
				Q.pop();
				if (grid[x][y] == 'B' && (deg_left_right[x][y] == 1 || deg_top_bottom[x][y] == 1)) {
					bool found = false;
					for (int d=0; d<4; ++d)
						if (((d&1) && deg_left_right[x][y] == 1 && !has_left_right[x][y]) || (!(d&1) && deg_top_bottom[x][y] == 1 && !has_top_bottom[x][y])) {
							int tx = x + dx[d];
							int ty = y + dy[d];
							if (tx >= 0 && tx < n && ty >= 0 && ty < m && grid[tx][ty] == 'W') {
								if (d&1)
									has_left_right[x][y] = true;
								else
									has_top_bottom[x][y] = true;
								found = true;
								grid[tx][ty] = '.';
								deg_left_right[tx][ty] = deg_top_bottom[tx][ty] = 0;
								for (int d2=0; d2<4; ++d2) {
									int tx2 = tx + dx[d2];
									int ty2 = ty + dy[d2];
									if (tx2 >= 0 && tx2 < n && ty2 >= 0 && ty2 < m && grid[tx2][ty2] == 'B') {
										if (d2&1) {
											--deg_left_right[tx2][ty2];
											if (deg_left_right[tx2][ty2] == 1)
												Q.push(tx2*m + ty2);
										}
										else {
											--deg_top_bottom[tx2][ty2];
											if (deg_top_bottom[tx2][ty2] == 1)
												Q.push(tx2 * m + ty2);
										}
									}
								}
							}
						}
					assert(found);
					if (has_left_right[x][y])
						assert(deg_left_right[x][y] <= 0);
					if (has_top_bottom[x][y])
						assert(deg_top_bottom[x][y] <= 0);
					if (has_left_right[x][y] != has_top_bottom[x][y])
						Q2.push(x * m + y);
				}
				else if (grid[x][y] == 'W' && deg_left_right[x][y] + deg_top_bottom[x][y] == 1) {
					grid[x][y] = '.';
					bool found = false;
					for (int d=0; d<4; ++d) {
						int tx = x + dx[d];
						int ty = y + dy[d];
						if (tx >= 0 && tx < n && ty >= 0 && ty < m && grid[tx][ty] == 'B' && ((d&1) && !has_left_right[tx][ty] || !(d&1) && !has_top_bottom[tx][ty])) {
							assert(!found);
							found = true;
							if (d&1) {
								has_left_right[tx][ty] = true;
								deg_left_right[tx][ty] = 0;
							}
							else {
								has_top_bottom[tx][ty] = true;
								deg_top_bottom[tx][ty] = 0;
							}
							if (has_left_right[tx][ty] != has_top_bottom[tx][ty])
								Q2.push(tx * m + ty);
							for (int d2=d&1; d2<4; d2 += 2) {
								int tx2 = tx + dx[d2];
								int ty2 = ty + dy[d2];
								if (tx2 >= 0 && tx2 < n && ty2 >= 0 && ty2 < m && grid[tx2][ty2] == 'W') {
									if (d2&1) {
										--deg_left_right[tx2][ty2];
										assert(deg_left_right[tx2][ty2] >= 0);
										if (deg_left_right[tx2][ty2] + deg_top_bottom[tx2][ty2] == 1)
											Q.push(tx2 * m + ty2);
									}
									else {
										--deg_top_bottom[tx2][ty2];
										assert(deg_top_bottom[tx2][ty2] >= 0);
										if (deg_left_right[tx2][ty2] + deg_top_bottom[tx2][ty2] == 1)
											Q.push(tx2 * m + ty2);
									}
								}
							}
						}
					}
					assert(found);
				}
			}
			// select one of the "half-finished" B and arbitrarily do the decision for the other half
			while(!Q2.empty()) {
				int i = Q2.front() / m;
				int j = Q2.front() % m;
				Q2.pop();
				if (has_left_right[i][j] != has_top_bottom[i][j]) {
					if (deg_left_right[i][j] == 2) {
						--deg_left_right[i][j]; // remove one possibility from the B square
						--deg_left_right[i][j-1]; // remove the corresponding possibility from the W square
						Q.push(i*m + j);
						if (deg_top_bottom[i][j-1] + deg_left_right[i][j-1] == 1)
							Q.push(i * m + j - 1);
						break;
					}
					if (deg_top_bottom[i][j] == 2) {
						--deg_top_bottom[i][j]; // remove one possibility from the B square
						--deg_top_bottom[i-1][j]; // remove one possibility from the W square
						Q.push(i*m + j);
						if (deg_top_bottom[i-1][j] + deg_left_right[i-1][j] == 1)
							Q.push((i-1) * m + j);
						break;
					}
				}
			}
			if (Q.empty())
				break;
		}
		bool valid = true;
		for (int i=0; i<n; ++i) {
			for (int j=0; j<m; ++j) {
				if (grid[i][j] == 'B' && has_left_right[i][j] && has_top_bottom[i][j])
					grid[i][j] = '.';
				if (grid[i][j] != '.')
					valid = false;
			}
		}
		puts(valid?"YES":"NO");
	}
	return 0;
}
