/*
 *   [NWERC'11] Piece It Together
 *   by: Jan Kuipers
 */

#include <iostream>
#include <vector>
#include <string>

using namespace std;

const int dy[4] = { 0, 0,-1,+1};
const int dx[4] = {-1,+1, 0, 0};

int Y,X,N;
vector<string> s;
vector<vector<int> > c;
vector<int> v;
vector<int> changed;

bool consistent (int n, int vn) {

	// check consistency
	if (v[n]!=-1) return v[n]==vn;
	
	v[n] = vn;
	changed.push_back(n);

	// v[n]=0, so v[c[n,i]]=1 for all i
	if (vn==0) {
		for (int i=0; i<c[n].size(); i++)
			if (!consistent(c[n][i],1)) 
				return false;
	}

	// v[!n]=!v[n]
	if (!consistent(n+(n%2==0?1:-1),1-vn)) 
		return false;

	return true;
}

bool possible () {

	// number of white equals twice number of black
	int B=0,W=0;
	for (int y=0; y<Y; y++)
		for (int x=0; x<X; x++) {
			if (s[y][x]=='W') W++;
			if (s[y][x]=='B') B++;
		}

	if (W != 2*B) return false;

	N = 4*Y*X;
	c = vector<vector<int> >(N);

	// B connected to left or right and not to both
	for (int y=1; y+1<Y; y++)
		for (int x=1; x+1<X; x++)
			if (s[y][x]=='B') {
				int i1 = 2*(y*X+x-1);
				int i2 = 2*(y*X+x);
				c[i1].push_back(i2);
				c[i2].push_back(i1);
				c[i1+1].push_back(i2+1);
				c[i2+1].push_back(i1+1);
			}

	// B connected to upper or lower and not to both
	for (int y=1; y+1<Y; y++)
		for (int x=1; x+1<X; x++)
			if (s[y][x]=='B') {
				int i1 = 2*(Y*X+y*X-X+x);
				int i2 = 2*(Y*X+y*X+x);
				c[i1].push_back(i2);
				c[i2].push_back(i1);
				c[i1+1].push_back(i2+1);
				c[i2+1].push_back(i1+1);
			}

	// no W connected to multiple squares
	for (int y=1; y+1<Y; y++)
		for (int x=1; x+1<X; x++)
			if (s[y][x]=='W')
				for (int d1=0; d1<4; d1++) {
					int y1=y+dy[d1];
					int x1=x+dx[d1];
					if (y1>=0 && y1<Y && x1>=0 && x1<X)
						for (int d2=d1+1; d2<4; d2++) {
							int y2=y+dy[d2];
							int x2=x+dx[d2];
							if (y2>=0 && y2<Y && x2>=0 && x2<X) {
								int i1 = 2*(min(y1,y)*X+min(x1,x) + (d1<2?0:Y*X));
								int i2 = 2*(min(y2,y)*X+min(x2,x) + (d2<2?0:Y*X));
								c[i1+1].push_back(i2+1);
								c[i2+1].push_back(i1+1);
							}
						}
				}
	v = vector<int>(N,-1);
	
	// equal colored and empty squares are not connected
	for (int y=0; y<Y; y++)
		for (int x=0; x<X; x++) {
			if (x+1<X && (s[y][x]=='.' || s[y][x+1]=='.' || s[y][x]==s[y][x+1]))
				if (!consistent(2*(y*X+x),0)) return false;
			if (y+1<Y && (s[y][x]=='.' || s[y+1][x]=='.' || s[y][x]==s[y+1][x]))
				if (!consistent(2*(Y*X+y*X+x),0)) return false;
		}
	
	// connect the remaining squares
	for (int i=0; i<N; i++) {
		changed.clear();
		if (consistent(i,0)) continue;
		
		for (int j=0; j<changed.size(); j++)
			v[changed[j]]=-1;
		
		changed.clear();
		if (!consistent(i,1)) return false;
	}
	
	return true;
}

int main () {

  int runs;
  cin>>runs;

  while (runs--) {

		cin >> Y >> X;
		Y+=2;	X+=2;
		
		s = vector<string>(Y);
		s[0] = string(X,'.');
		for (int y=1; y<Y-1; y++) {
			cin >> s[y];
			s[y] = "." + s[y] + ".";
		}
		s[Y-1] = string(X,'.');
		
		cout << (possible() ? "YES" : "NO") << endl;		
  }
	
  return 0;   
}
