import java.io.BufferedInputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

/**
 *   [NWERC'11] Smoking Gun
 *   by: yvind Grotmol
 */
public final class oyvind {

	static Scanner in = new Scanner(new BufferedInputStream(System.in));

	public static void main(String[] args) {
		for (int T = in.nextInt(); T --> 0; ) {
			new oyvind().go();
		}
	}
	
	static final double SENTINEL = -1e300;
	
	static final double EPS = 5e-7;
	
	static boolean isGreaterThanZero(double value) {
		if (-EPS < value && value < EPS) {
			String message = "The margin of this distance calculation is less than " + EPS;
			message += "; the value checked against zero was just " + value;
			throw new RuntimeException(message);
		}
		return value > 0;
	}

	void go() {
		int M = in.nextInt();
		int N = in.nextInt();
		
		String[] ID = new String[M];
		Map<String, Integer> INDEX = new HashMap<String, Integer>();
		double[] X = new double[M];
		double[] Y = new double[M];
		double[][] G = new double[M][M];
		boolean[] fired = new boolean[M];
		
		// Read first part of input, initialize
		for (int i=0; i<M; ++i) {
			ID[i] = in.next();
			X[i] = in.nextInt();
			Y[i] = in.nextInt();
			INDEX.put(ID[i], i);
			Arrays.fill(G[i], SENTINEL);
		}
		
		// Read second part of input
		for (int i=0; i<N; ++i) {
			String[] line;
			do {
				line = in.nextLine().split(" ");
			} while (line.length < 6);
			int a = INDEX.get(line[2]);
			int b = INDEX.get(line[5]);
			int c = INDEX.get(line[0]);
			// Calculate the difference between the two distances.
			// Note that the actual speed of sound does not matter.
			double margin = Math.sqrt((X[a]-X[c])*(X[a]-X[c])+(Y[a]-Y[c])*(Y[a]-Y[c]))
			              - Math.sqrt((X[b]-X[c])*(X[b]-X[c])+(Y[b]-Y[c])*(Y[b]-Y[c]));
			if (margin > G[a][b])
				G[a][b] = margin;
			fired[a] = fired[b] = true;
		}
		
		// Floyd-Warshall to find longest path between all nodes
		for (int k=0; k<M; ++k) {
			for (int a=0; a<M; ++a) {
				if (G[a][k] == SENTINEL) continue;
				for (int b=0; b<M; ++b) {
					if (G[k][b] == SENTINEL) continue;
					if (G[a][b] < G[a][k] + G[k][b]) {
						G[a][b] = G[a][k] + G[k][b];
					}
				}
			}
		}
		
		// If we have a loop with positive sum along the edges,
		// then the witnesses cannot all be right
		for (int a=0; a<M; ++a) {
			if (isGreaterThanZero(G[a][a])) {
				System.out.println("IMPOSSIBLE");
				return;
			}
		}

		// Do a DAG sorting of the shooters.
		// Inefficient implementation (n^3),
		// but doesn't matter because Floyd-Warshall is the same.
		int numShooters = 0;
		for (boolean b : fired) {
			if (b) ++numShooters;
		}
		boolean[] used = new boolean[M];
		StringBuilder sb = new StringBuilder();
		for (int i=0; i<numShooters; ++i) {
			int candidate = -1;
			for (int a=0; a<M; ++a) {
				if (!fired[a] || used[a]) continue;
				boolean ok = true;
				for (int b=0; b<M; ++b) {
					if (!fired[b] || used[b]) continue;
					if (isGreaterThanZero(G[b][a])) {
						ok = false;
						break;
					}
				}
				if (ok) {
					// If there are multiple possibilities,
					// then the DAG sorting only gives a partial ordering,
					// and we do not know the exact sequence of shots fired.
					if (candidate != -1) {
						System.out.println("UNKNOWN");
						return;
					}
					candidate = a;
				}
			}
			if (candidate == -1) {
				throw new RuntimeException("Not supposed to happen...");
			}
			if (sb.length() > 0) sb.append(' ');
			sb.append(ID[candidate]);
			used[candidate] = true;
		}
		System.out.println(sb);
	}

}
