/*
 *   [NWERC'11] Random Storage
 *   by: Jan Kuipers
 */

#include <iostream>
#include <vector>
#include <set>
#include <cmath>

using namespace std;

struct point {
  int x,y;
  point(int _x=0, int _y=0): x(_x), y(_y) {}
};

struct line {
  point p,q;
  line (point _p=point(), point _q=point()): p(_p), q(_q) {}
};

int sgn (int a) { return a==0 ? 0 : a>0 ? 1 : -1; }
int operator^ (point a, point b) { return a.x*b.y - b.x*a.y; }
int operator* (point a, point b) { return a.x*b.x + a.y*b.y; }
bool operator< (point a, point b) { return a.x!=b.x ? a.x<b.x : a.y<b.y; }
point operator- (point a, point b) { return point(a.x-b.x,a.y-b.y); }
  
bool intersect (line a, line b) {

  point da = a.q-a.p;
  point db = b.q-b.p;

  if ((da.x==0 && da.y==0) || (db.x==0 && db.y==0))
    return false;
  
  if ((da^db) != 0) {
    return sgn(da^(b.q-a.p)) * sgn(da^(b.p-a.p)) <= 0 && 
           sgn(db^(a.q-b.p)) * sgn(db^(a.p-b.p)) <= 0;
  }
  else {
    if ((da^(b.p-a.p)) != 0) return false;
    
    int a1 = a.p * da;
    int a2 = a.q * da;
    if (a1>a2) swap(a1,a2);

    int b1 = b.p * da;
    int b2 = b.q * da;
    if (b1>b2) swap(b1,b2);

    if (a1==b1) return true;
      
    if (a1>b1) {
      swap(a1,b1);
      swap(a2,b2);
    }
    
    return a2>=b1;
  }  
}

int main () {

  int runs;
  cin >> runs;

  while (runs--) {

    int S,R,W,P;
    cin >> S >> R >> W >> P;

    set<point> alls;

    for (int si=0; si<S; si++) {
      point s;
      cin >> s.x >> s.y;
      alls.insert(s);
    }

    vector<line> w(W);
    for (int wi=0; wi<W; wi++) 
      cin >> w[wi].p.x >> w[wi].p.y >> w[wi].q.x >> w[wi].q.y;
    
    for (int pi=0; pi<P; pi++) {

      vector<point> res;
    
      point p,s;
      cin >> p.x >> p.y;

      for (s.x=p.x-R; s.x<=p.x+R; s.x++)
	for (s.y=p.y-R; s.y<=p.y+R; s.y++) {
	  double D = sqrt((s-p)*(s-p));
	  
	  if (D<=R && alls.count(s)) {
	    for (int wi=0; wi<W; wi++) 
	      if (intersect(line(s,p),w[wi])) D+=1;
	    if (D<=R)
	      res.push_back(s);
	  }
	}

      cout << res.size();
      for (int i=0; i<res.size(); i++)
	cout << " (" << res[i].x << "," << res[i].y << ")";
      cout << endl;
    }
  }
  
  return 0;
}
